package carona;

import java.util.ArrayList;
import java.util.stream.Collectors;

import caronante.Caronante;
import caroneiro.Caroneiro;
import pagamento.MetodoPagamento;
import utilidades.HelperFormatacao;

public class Carona {
	private final ArrayList<Caroneiro> caroneiros;
	private final Caronante caronante;
	private double latitudeEncontro;
	private double longitudeEncontro;
	private double latitudeDestino;
	private double longitudeDestino;
	private String horaDiaEncontro;
	private int ocupacaoMaxima;
	private float valor;
	private final ArrayList<MetodoPagamento> formaPagAceitas;
	
	public Carona(Caronante caronante, double latitudeEncontro, double longitudeEncontro,
			double latitudeDestino, double longitudeDestino, String horaDiaEncontro, float valor) {
		this.ocupacaoMaxima = caronante.getAssentosDisponiveis();
		this.caroneiros = new ArrayList<Caroneiro>();
		this.caronante = caronante;
		this.latitudeEncontro = latitudeEncontro;
		this.longitudeEncontro = longitudeEncontro;
		this.latitudeDestino = latitudeDestino;
		this.longitudeDestino = longitudeDestino;
		this.horaDiaEncontro = horaDiaEncontro;		
		this.formaPagAceitas = new ArrayList<MetodoPagamento>();
	}
	
	public Carona(Caronante caronante, double latitudeEncontro, double longitudeEncontro,
			double latitudeDestino, double longitudeDestino, String horaDiaEncontro, float valor, int ocupacaoMaxima) {
		this(caronante, latitudeEncontro, longitudeEncontro, latitudeDestino, longitudeDestino, horaDiaEncontro, valor);
		this.ocupacaoMaxima = ocupacaoMaxima;
	}
	
	public float getValor() {
		return valor;
	}

	public void setValor(float valor) {
		this.valor = valor;
	}

	public double getLatitudeEncontro() {
		return latitudeEncontro;
	}
	public void setLatitudeEncontro(double latitudeEncontro) {
		this.latitudeEncontro = latitudeEncontro;
	}
	public double getLongitudeEncontro() {
		return longitudeEncontro;
	}
	public void setLongitudeEncontro(double longitudeEncontro) {
		this.longitudeEncontro = longitudeEncontro;
	}
	public double getLatitudeDestino() {
		return latitudeDestino;
	}
	public void setLatitudeDestino(double latitudeDestino) {
		this.latitudeDestino = latitudeDestino;
	}
	public double getLongitudeDestino() {
		return longitudeDestino;
	}
	public void setLongitudeDestino(double longitudeDestino) {
		this.longitudeDestino = longitudeDestino;
	}
	public String getHoraDiaEncontro() {
		return horaDiaEncontro;
	}
	public void setHoraDiaEncontro(String horaDiaEncontro) {
		this.horaDiaEncontro = horaDiaEncontro;
	}
	public int getOcupacaoMaxima() {
		return ocupacaoMaxima;
	}

	public int getAssentosDisponiveis() {
		return ocupacaoMaxima - caroneiros.size();
	}
	
	public int verificaOcupacao() {
		return caroneiros.size();
	}
	
	public boolean caronaVazia() {
		return caroneiros.isEmpty();
	}
	
	public boolean adicionarCaroneiro(Caroneiro caroneiro) {
		if (getAssentosDisponiveis() == 0) {
			return false;
		}
		
		//Coloca na primeira posição vazia. Isso é suficiente pois não há um removerCaroneiro
		caroneiros.add(caroneiro);
		
		return true;
	}
	
	public boolean removerCaroneiro(Caroneiro caroneiro) {
		//Precisa fazer isso pra remover por id. É feio, e dá pra melhorar fazendo sobrescrita do Equals
		//pra considerar id. Mas não entra no escopo da matéria até o momento, então assim é o melhor por agora
		for(Caroneiro caroneiroBuscado: caroneiros) {
			if(caroneiroBuscado.getPerfil().getUsuario().getId() == caroneiro.getPerfil().getUsuario().getId()) {
				caroneiros.remove(caroneiroBuscado);
				return true; //Importante sair porque, se não estou enganado, remover um elemento invalida o iterador
			}
		}
		return false;
	}
	
	public boolean adicionarFormaPagamento(MetodoPagamento metodoPagamento) {
		if (formaPagAceitas.contains(metodoPagamento)) {
			return false;
		}
		
		formaPagAceitas.add(metodoPagamento);
			
		return true;
	}
	
	public boolean removerFormaPagamento(MetodoPagamento metodoPagamento) {
		return formaPagAceitas.remove(metodoPagamento);
	}
	
	public boolean checarExistenciaFormaPagamento(MetodoPagamento metodoPagamento) {
		return formaPagAceitas.contains(metodoPagamento);
	}
	
	public boolean caronaGratuita() {
		return formaPagAceitas.contains(MetodoPagamento.Gratis);
	}
	
	public String toString(int numeroEspacos) {
		String espacos = HelperFormatacao.criaEspacos(numeroEspacos);
		String stringCaroneiros = espacos + "*Nao ha caroneiros nessa carona*";
		if(!caroneiros.isEmpty()) {
			StringBuilder stringBuilder = new StringBuilder();
			for(Caroneiro caroneiro: caroneiros) {			
				stringBuilder.append(caroneiro.toStringDesdeUsuario(numeroEspacos + 1));
			}
			
			stringCaroneiros = stringBuilder.toString();			
		}
		
		String stringFormaPag = espacos + "*Nao ha forma de pagamento cadastrado*";
		if(!formaPagAceitas.isEmpty()) {
			//Um exemplo com stream. Nesse caso ele faz um stream (simplificando, nesse caso é como se fosse uma lista)
			//com as formas de pagamento, e então transforma cada elemento MetodoPagamento para uma string que
			//tem o retorno do método "name" (que retorna o string correspondente ao enum) do enum.
			//Finalmente, dá um join com ", "
			stringFormaPag = formaPagAceitas.stream()
				.map(MetodoPagamento::name)
				.collect(Collectors.joining(", "));			
		}	
		
		return String.format("%s| Carona\n"
				+ "%s| - Coordenadas do encontro: (%f, %f)\n"
				+ "%s| - Coordenadas do destino: (%f, %f)\n"
				+ "%s| - Hora e dia do encontro: %s\n"
				+ "%s| - Ocupacao maxima: %d\n"
				+ "%s| - Numero de assentos disponiveis: %d\n"
				+ "%s| - Dados do caronante: \n%s"
				+ "%s| - Dados dos caroneiros: \n%s"
				+ "%s| - Formas de pagamento aceitas: %s\n",
				espacos, espacos, this.latitudeEncontro, this.longitudeEncontro,
				espacos, this.latitudeDestino, this.longitudeDestino,
				espacos, this.horaDiaEncontro,
				espacos, this.ocupacaoMaxima,
				espacos, this.getAssentosDisponiveis(),
				espacos, this.caronante.toStringDesdeUsuario(numeroEspacos + 1),
				espacos, stringCaroneiros,
				espacos, stringFormaPag);
	}
	
	@Override
	public String toString() {
		return toString(0);
	}

	
	
}
