/*
 * Respostas:
 * 
 * 1 - A vantagem é melhor acesso das variáveis. A desvantagem é o alto acoplamento, o que é algo que deve ser
 *  evitado sempre que possível. Além disso, algumas vezes um relacionamento bi-direcional é a solução menos pior.
 *  
 * 2 - Não, poderíamos simplesmente ter passado a avaliação para Carona. Ainda teria somente um valor, e seria de
 *  fácil acesso. Uma vantagem é que usaria uma classe a menos, e facilitaria o acesso entre as classes. Uma
 *  desvantagem é que isso sairia do padrão do resto do projeto, e consistência é importante para o entendimento.
 *  Além disso, cada classe deve ter uma função bem específica, quebrar em mais pedaços costuma ser bom pra não
 *  deixar alguma classe com muita responsabilidade.
 *  
 * 3 - Não são. É possível setar, por exemplo, em CaronaCaronante ou CaronaCaroneiro, só uma das variáveis
 *  sem modificar o resto dos relacionamentos, o que geraria uma inconsistência.
 *  Entre as formas de resolver o problema estão:
 *   - Usar algum framework que cuida disso para você, (frameworks web em geral cuidam quando tem algo chamado ORM,
 *    ruby on rails por exemplo);
 *   - Não usar relacionamentos bi-direcionais, de alguma forma, então só há a relação de um lado (não vejo como
 *   possível nesse caso).
 *   - Fazer com que os relacionamentos sejam criados de forma "atômica". Então se em algum lugar é possível mudar
 *   alguma parte de um relacionamento (por exemplo, mudar a referência a um CaronaCaronante em Carona), então 
 *   necessariamente o método precisa mudar os outros relacionamentos de acordo também. Isso não é trivial pois,
 *   em relacionamentos bi-direcionais, não é possível ter todas as referências do relacionamento ao criá-lo, então
 *   em algum momento, mesmo que pequeno, deve haver uma inconsistência. Mas é possível limitar a inconsistência para
 *   que só seja possível ser inconsistente se um dos valores for nulo, por exemplo.
 */

package inicio;

import carona.Carona;
import caronante.Caronante;
import caroneiro.Caroneiro;
import grupo.Grupo;
import grupo.GrupoPrivado;
import grupo.GrupoPublico;
import pagamento.MetodoPagamento;
import perfil.Perfil;
import usuario.Usuario;

public class Lab6 {

	public static void main(String[] args) {
		Caronante caronante = new Caronante(5, "Chamber pop", "abcd-123", "abdef12345",
				"Alguma marca", "Algum modelo", 3);
		Caroneiro caroneiro1 = new Caroneiro(true);
		Caroneiro caroneiro2 = new Caroneiro(false);
		Caroneiro caroneiro3 = new Caroneiro("5435345345");
		
		Perfil perfilCaronante = new Perfil('f',"01/01/2001", "Campinas", "SP", "123123", true, caronante, null, null);		
		Usuario usuario1 = new Usuario("Joanna", "jnewsom@dunno.com", "sadie", true, perfilCaronante);
		perfilCaronante.setUsuario(usuario1);
		
		Perfil perfilCaroneiro1 = new Perfil('m',"01/01/1999", "Sorocaba", "SP", "123123", true, null, caroneiro1, null);		
		Usuario usuario2 = new Usuario("Sufjan", "sufjan@dunno.com", "wasps", true, perfilCaroneiro1);
		perfilCaroneiro1.setUsuario(usuario2);
		
		Perfil perfilCaroneiro2 = new Perfil('m',"01/01/1990", "Sao Paulo", "SP", "123123", true, null, caroneiro2, null);		
		Usuario usuario3 = new Usuario("Shinji", "hikari@dunno.com", "something", true, perfilCaroneiro2);
		perfilCaroneiro2.setUsuario(usuario3);
		
		Perfil perfilCaroneiro3 = new Perfil('f',"01/01/1999", "Belo Horizonte", "MG", "123123", true, null, caroneiro3, null);		
		Usuario usuario4 = new Usuario("Lily Briscoe", "lily@dunno.com", "ramsay", true, perfilCaroneiro3);
		perfilCaroneiro3.setUsuario(usuario4);
		
		Grupo grupo = new Grupo("Musicos", "Alguma descricao");
		usuario1.adicionarGrupo(grupo);
		usuario2.adicionarGrupo(grupo);
		
		GrupoPrivado grupoPrivado = new GrupoPrivado("Pilotos de robos", "Hm");
		usuario3.adicionarGrupo(grupoPrivado);
		
		GrupoPublico grupoPublico = new GrupoPublico("Pintores", "Alguem?");
		usuario4.adicionarGrupo(grupoPublico);
		
		//Verifica funcionamento do método
		Carona carona = caronante.oferecerCarona(13.2, 15.46, 13.45, 15.23, "15:05:00 10/10/2018", 20.50f);
		
		caroneiro1.pedirCarona(carona);
		caroneiro2.pedirCarona(carona);
		caroneiro3.pedirCarona(carona);
		
		carona.adicionarFormaPagamento(MetodoPagamento.Gratis);
		
		System.out.println("Testando a atribuição de avaliacoes");
		System.out.println("Caronante dando a nota: " + carona.atribuirNotaCaronante(8));
		System.out.println("Caroneiro 1 dando a nota: " + carona.atribuirNotaCaroneiro(1, 3));
		System.out.println("Caroneiro 2 dando a nota: " + carona.atribuirNotaCaroneiro(2, 7.5f));
		System.out.println("Caroneiro 3 dando a nota: " + carona.atribuirNotaCaroneiro(3, 10));
		System.out.println("Caroneiro 4 dando a nota (inexistente): " + carona.atribuirNotaCaroneiro(4, 10));		
		
		//Do jeito que foi feito até agora, o print funciona normalmente
		System.out.println(carona);
		
	}

}
